package org.steveshipway.cheshirecat;

import java.io.File;
import java.util.Collections;
import java.util.HashMap;
import java.util.Map;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraftforge.common.MinecraftForge;

import org.apache.logging.log4j.Logger;
import org.steveshipway.cheshirecat.lib.Constants;

import cpw.mods.fml.common.Mod;
import cpw.mods.fml.common.Mod.EventHandler;
import cpw.mods.fml.common.Mod.Instance;
import cpw.mods.fml.common.Mod.Metadata;
import cpw.mods.fml.common.ModMetadata;
import cpw.mods.fml.common.SidedProxy;
import cpw.mods.fml.common.event.FMLInitializationEvent;
import cpw.mods.fml.common.event.FMLPostInitializationEvent;
import cpw.mods.fml.common.event.FMLPreInitializationEvent;
import cpw.mods.fml.common.event.FMLServerStartingEvent;

@Mod(modid = Constants.MODID, name = Constants.MODNAME, version = Constants.VERSION,
		useMetadata = true)
public class CheshireCat
{    
   
	@Metadata(value = Constants.MODID)
	public static ModMetadata metadata;
	
	@SidedProxy(clientSide = "org.steveshipway.cheshirecat.ClientProxy", 
			serverSide = "org.steveshipway.cheshirecat.CommonProxy")
	public static CommonProxy proxy;

	// scan for all defined blueprints
	public static Portfolio portfolio;
	public static String bppath = null;
	public static Logger logger;
	
	public static void loadPortfolio() {
    	portfolio.load( bppath );
	}
	
    @Mod.EventHandler
    public void preInit(FMLPreInitializationEvent event) {
    	logger = event.getModLog();
    	logger.info("Starting Cheshire Cat Mod : Ver " + Constants.VERSION);
    	portfolio = new Portfolio(logger);
    	bppath = event.getModConfigurationDirectory() + File.separator + "bp";
    	File bpDir = new File(bppath);
    	if( !bpDir.exists() ) { bpDir.mkdir(); }
    	loadPortfolio();
    	proxy.preInit(event);
    }
 
    @Mod.EventHandler
    public void init(FMLInitializationEvent event) {
    	proxy.init(event);
    	// special client renderers
    	proxy.registerRenderers();
    }
 
    @Mod.EventHandler
    public void postInit(FMLPostInitializationEvent event) {
    	proxy.postInit(event);
    }
    
    @Mod.EventHandler
    public void serverLoad(FMLServerStartingEvent event)
    {
        logger.info("Loading server console commands.");
        proxy.serverLoad(event);
    }
    
    // identify block type, and return correct DV to rotate by specified amount of quarter-turns
    public static int rotateBlock(Block b,int dv, int r) {
    	int newDv = dv;
        final Map<Integer, Integer> rotate = new HashMap<Integer, Integer>();

    	while( r > 3 ) { r -= 4; }
    	while( r < 0 ) { r += 4; }
    	if( r == 0 ) { return dv; } // null operation

    	// only certain blocks have orientation, and some have only 2 relevant orientations.
    	if( b == Blocks.log || b == Blocks.log2 ) {
    		// only need quarter-turns, and only for horizontal alignments
    		// swap the top two bits
    		if( r == 1 || r == 3) {
    			if( newDv >= 4 && newDv <= 7 ) {
    				newDv += 4;
    			} else if( newDv >= 8 && newDv <= 11) {
    				newDv -= 4;
    			}
    		}
    		return newDv;
    	} else if( b == Blocks.torch || b == Blocks.redstone_torch 
    			|| b == Blocks.stone_button || b == Blocks.wooden_button
    	) {
    		// Standard 1-3-2-4 rotation, do not rotate 0 and 5, preserve top bit
            rotate.put(1, 3); rotate.put(2, 4);
            rotate.put(3, 2); rotate.put(4, 1);
            rotate.put(5, 5); rotate.put(0, 0);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); }
    		return newDv;
    	} else if( b == Blocks.lever ) {
    		// Standard 1-3-2-4 rotation, 5/6 and 7/0 swap, preserve top bit
            rotate.put(1, 3); rotate.put(2, 4);
            rotate.put(3, 2); rotate.put(4, 1);
            rotate.put(5, 6); rotate.put(6, 5);
            rotate.put(7, 0); rotate.put(0, 7);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); }
    		return newDv;
    	} else if( b == Blocks.bed || b == Blocks.lit_pumpkin || b == Blocks.pumpkin 
    			|| b == Blocks.powered_repeater || b == Blocks.unpowered_repeater
    			|| b == Blocks.unpowered_comparator || b == Blocks.powered_comparator 
    			|| b == Blocks.trapdoor || b == Blocks.end_portal_frame
    			|| b == Blocks.fence_gate 
    			|| b == Blocks.cocoa || b == Blocks.tripwire_hook
    			|| b == Blocks.anvil ) {
    		// 0-1-2-3 rotation preserving high bits
    		int i = (( newDv & 3 ) + r ) & 3;
    		newDv = (newDv & 12) | i;
    		return newDv;
    	} else if( b == Blocks.piston || b == Blocks.piston_head || b == Blocks.piston_extension  || b == Blocks.sticky_piston
    			|| b == Blocks.cobblestone_wall 
    			|| b == Blocks.activator_rail || b == Blocks.detector_rail || b == Blocks.golden_rail
    			|| b == Blocks.ladder || b == Blocks.chest || b == Blocks.ender_chest || b == Blocks.trapped_chest  
    			|| b == Blocks.furnace || b == Blocks.lit_furnace
    			|| b == Blocks.wall_sign || b == Blocks.dropper || b == Blocks.hopper || b == Blocks.dispenser
    			|| b == Blocks.skull
    			) {
    		// 2-5-3-4 rotation preserving top bit
            rotate.put(2, 5); rotate.put(5, 3);
            rotate.put(3, 4); rotate.put(4, 2);
            rotate.put(0, 0); rotate.put(1, 1);
    		int i = r;
    		//logger.info("Block: " + Block.blockRegistry.getNameForObject(b) + "  Rot: " + r + "  DV: " + newDv);
    		while(i>0) { 
    			i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); 
    			//logger.info("Rotate -- DV: " + newDv);
    		}
    		return newDv;
    	//} else if( b == Blocks.skull ) {
    	//	// 2-4-3-5 rotation preserving top bit
        //    rotate.put(2, 4); rotate.put(4, 3);
        //    rotate.put(3, 5); rotate.put(5, 2);
        //    rotate.put(0, 0); rotate.put(1, 1);
    	//	int i = r;
    	//	while(i>0) { i-=1; newDv = rotate.get(newDv & 7) | (newDv & 8); }
    	//	return newDv;
    	} else if( b == Blocks.stone_stairs || b == Blocks.stone_brick_stairs || b == Blocks.acacia_stairs || b == Blocks.birch_stairs
    			|| b == Blocks.brick_stairs || b == Blocks.nether_brick_stairs || b == Blocks.dark_oak_stairs || b == Blocks.jungle_stairs
    			|| b == Blocks.oak_stairs || b == Blocks.quartz_stairs || b == Blocks.sandstone_stairs || b == Blocks.spruce_stairs ) {
    	 	// 3-0-2-1 stairs rotation preserving upper bits
            rotate.put(3, 0); rotate.put(0, 2);
            rotate.put(2, 1); rotate.put(1, 3);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv & 3) | (newDv & 12); }
    		return newDv;
    	} else if( b == Blocks.wooden_door || b == Blocks.iron_door  ) {
    		// 0-1-2-3 on bottom 2 bits if top bit unset
    		if( newDv > 7 ) { return newDv; }
    		int i = (( newDv & 3 ) + r ) & 3;
    		newDv = (newDv & 12) | i;
    		return newDv;
    	} else if( b == Blocks.rail ) {
    		// 2-5-3-4 with 0-1 and 6-7-8-9
            rotate.put(2, 5); rotate.put(5, 3);
            rotate.put(3, 4); rotate.put(4, 2);
            rotate.put(0, 1); rotate.put(1, 0);
            rotate.put(6, 7); rotate.put(7, 8);
            rotate.put(8, 9); rotate.put(9, 6);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }
    		return newDv;    		
        }   else if( b == Blocks.standing_sign ) {
        	newDv = ( newDv - 4 );  if( newDv < 0) { newDv += 15; }
        	return newDv;
        } else if( b == Blocks.brown_mushroom_block || b == Blocks.red_mushroom_block ) {
            rotate.put(1, 3); rotate.put(3, 9);
            rotate.put(9, 7); rotate.put(7, 1);
            rotate.put(2, 6); rotate.put(6, 8);
            rotate.put(8, 4); rotate.put(4, 2);
            rotate.put(0, 0); rotate.put(5, 5);
            rotate.put(10, 10); 
            rotate.put(14, 14); rotate.put(15, 15);
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }
    		return newDv;    		
        } else if( b == Blocks.vine ) {
    		int i = r;
    		while(i>0) { 
    			i-=1; 
            	newDv = newDv * 2;
            	if( newDv > 15 ) { newDv -= 15; } // rotate top bit to lowest 
    		}
    		return newDv;    		
        } else if( b == Blocks.quartz_block ) {
            rotate.put(4, 3); rotate.put(3, 4);
            rotate.put(0, 0); rotate.put(1, 1);
            rotate.put(2, 2); 
    		int i = r;
    		while(i>0) { i-=1; newDv = rotate.get(newDv); }
    		return newDv;    		
        }
    	
    	return newDv;
    }
}
