package org.steveshipway.cheshirecat.cmd;

import java.util.ArrayList;
import java.util.List;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.item.ItemWand;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.util.MathHelper;

public class BuildCommand implements ICommand {
	  private List aliases;
	  public BuildCommand()
	  {
	    this.aliases = new ArrayList();
	    this.aliases.add("build");
	  }

	  @Override
	  public String getCommandName()
	  {
	    return "build";
	  }

	  @Override
	  public String getCommandUsage(ICommandSender icommandsender)
	  {
	    return "build <blueprint> <x> <y> <z> <r>";
	  }

	  @Override
	  public List getCommandAliases()
	  {
	    return this.aliases;
	  }

	  @Override
	  public void processCommand(ICommandSender icommandsender, String[] astring)
	  {
		  int rotation = 0;
		  ChunkCoordinates location = new ChunkCoordinates();
	      if(astring.length < 1)
	      {
	    	  icommandsender.addChatMessage(new ChatComponentText("No blueprint name given"));
	    	  return;
	      }
	      // now we can test for a valid blueprint
	      if( ! CheshireCat.portfolio.defines(astring[0]) ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Invalid blueprint name '" + astring[0] + "'"));
	    	  return;	    	  
	      }
	      if(astring.length < 5)
	      {
	    	  icommandsender.addChatMessage(new ChatComponentText("Must give world coordinates, and rotation (0-3)"));
	    	  return;
	      }
	      try {
	      location.posX = Integer.parseInt(astring[1]);
	      location.posY = Integer.parseInt(astring[2]);
	      location.posZ = Integer.parseInt(astring[3]);
	      rotation = Integer.parseInt(astring[4]) & 3;
	      } catch( Exception e ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Invalid coordinates or rotation!"));
	    	  return;
	      }
	      if( CheshireCat.portfolio.build(icommandsender.getEntityWorld(), astring[0], location, rotation) ) {
	    	  icommandsender.addChatMessage(new ChatComponentText("Build successful"));
	      } else {
	    	  icommandsender.addChatMessage(new ChatComponentText("Build failed!"));
	      }
	  }

	  @Override
	  public boolean canCommandSenderUseCommand(ICommandSender icommandsender)
	  {
		  // is the player in create mode or holding the wand?
		  EntityPlayer player;
          
          if(icommandsender instanceof EntityPlayer){
                  player = (EntityPlayer)icommandsender;
          } else {
        	  return true;  // game console
          }
          if( player.capabilities.isCreativeMode ) {
        	  return true;  // player in creative mode
          }
          ItemStack item = player.getCurrentEquippedItem();
          if (item != null && item.getUnlocalizedName().endsWith(Constants.MODID + "_" + ItemWand.name) ) {
        	  return true;  // player holding Wand item
          }
          
		  return false;
	  }

	  @Override
	  public List addTabCompletionOptions(ICommandSender icommandsender,
	      String[] astring)
	  {
		  // should add the list of blueprints to them...
		  return CheshireCat.portfolio.list(astring[0]);
	  }

	  @Override
	  public boolean isUsernameIndex(String[] astring, int i)
	  {
		  // none of the parameters are usernames
   	      return false;
	  }

	  @Override
	  public int compareTo(Object o)
	  {
	    return 0;
	  }
}
