package org.steveshipway.cheshirecat.directive;

import java.util.Iterator;
import java.util.List;
import java.util.Random;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.steveshipway.cheshirecat.CheshireCat;

import net.minecraft.block.Block;
import net.minecraft.init.Blocks;
import net.minecraft.item.Item;
import net.minecraft.nbt.JsonToNBT;
import net.minecraft.nbt.NBTBase;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.nbt.NBTUtil;
import net.minecraft.server.MinecraftServer;
import net.minecraft.tileentity.TileEntity;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;
import net.minecraftforge.common.util.ForgeDirection;

public class BlockDirective extends Directive {
	// this is the main one!
	protected ChunkCoordinates location;
	protected boolean soft = false;
	protected Block block = null;
	protected String nbt = null; // possibly this should be a parsed NBT object
	protected int dv = 0;
	
	public BlockDirective(Block b,int dv, boolean soft,ChunkCoordinates offset, String nbt) {
		this.location = offset;
		this.block = b;
		this.soft = soft;
		this.nbt = nbt;
		this.dv = dv;
	}
	public BlockDirective(String definition) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*block)?\\s*(~?)(\\w+:\\w+)(?:\\((\\d+)\\))?\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)(\\s+\\{.*\\})?");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		Block b = null;
    		int dv = 0;
    		if( m.group(1).contains("~") ) {
    			this.soft = true;
    		}
    		// 2 is the block, 3 is the optional DV
    		// must identify the Block object from the m.group(2) text
    		b = Block.getBlockFromName(m.group(2));
    		if( b == null ) {
    			// invalid name
        		CheshireCat.logger.error("Invalid block name in BLOCK def: "+m.group(2));
    		} else {
    			if( m.group(3) != null ) {
        			dv = Integer.parseInt( m.group(3) );
        			// somehow, set the metadata on the block b
        		}
    			this.block = b;
    			this.dv = dv;
    			// coordinates
    			this.location = new  ChunkCoordinates();
    			this.location.posX = Integer.parseInt( m.group(4) );
    			this.location.posY = Integer.parseInt( m.group(5) );
    			this.location.posZ = Integer.parseInt( m.group(6) );
    			this.nbt = m.group(7);
    		}
    	} else {
    		CheshireCat.logger.error("Cannot parse BLOCK definition: "+definition);
    	}
	}
	
	@Override
	public boolean build(World world, ChunkCoordinates location, int r) {
		boolean rv;
		int newDv = dv;
		ChunkCoordinates blockPos = new ChunkCoordinates();
		
		if( block == null ) {
    		CheshireCat.logger.error("No valid block definition in BLOCK directive");
			return false;
		}
		if( world.isRemote ) {
    		CheshireCat.logger.error("Block placement called with remote world?  Should never happen.");
			return false;
		}
		// get block position
		blockPos = rotateAndAdd(this.location,r,location);
		newDv = CheshireCat.rotateBlock(this.block,dv,r);

		// place the block(s), possibly with the NBT in a tileentity
		//CheshireCat.logger.info("Base location  : "+location.posX+","+location.posY+","+location.posZ+" r="+r);
		//CheshireCat.logger.info("Offset         : "+this.location.posX+","+this.location.posY+","+this.location.posZ);
		//CheshireCat.logger.info("Block placement: "+blockPos.posX+","+blockPos.posY+","+blockPos.posZ);
		// 
		Block curBlock = world.getBlock(blockPos.posX, blockPos.posY, blockPos.posZ);
		int curDv = world.getBlockMetadata(blockPos.posX, blockPos.posY, blockPos.posZ);
		if(this.soft) {
			if( curBlock != Blocks.air && curBlock != Blocks.water && curBlock != Blocks.flowing_water ) {
				return true;  // soft block only placed in air or water
			}
		}
		
		if( curBlock != this.block ) {
			rv = world.setBlock(blockPos.posX,blockPos.posY,blockPos.posZ,this.block,newDv,2);
			if( !rv ) {
				CheshireCat.logger.warn("Failed to place block in the world");		
				return false;
			} else {
				CheshireCat.logger.warn("Placed block "+Block.blockRegistry.getNameForObject(this.block) +"  DV="+newDv);		
				
			}
		} else if( curDv != this.dv ) { // right block, but wrong DV
			rv = world.setBlockMetadataWithNotify(blockPos.posX,blockPos.posY,blockPos.posZ, newDv, 2);			
		}
		
		// we now use our own function to rotate the DV before setting.  This is because the block could fall off the wall
		// before the rotation was completed in some cases.
		
		// rotate the block r times
		// why not use a loop?  Well, because we can optimise, and 3 rotations can break things that fall off walls
		// there doesnt seem to be a way to specify how many 1/4 turns to rotate in one go
		//if( r == 1 ) {
		//	rv = this.block.rotateBlock(world, blockPos.posX,blockPos.posY,blockPos.posZ, ForgeDirection.UP);
		//} else if( r == 2 ) {
		//	rv = this.block.rotateBlock(world, blockPos.posX,blockPos.posY,blockPos.posZ, ForgeDirection.UP);
		//	rv = this.block.rotateBlock(world, blockPos.posX,blockPos.posY,blockPos.posZ, ForgeDirection.UP);
		//} else if( r == 3) {
		//	rv = this.block.rotateBlock(world, blockPos.posX,blockPos.posY,blockPos.posZ, ForgeDirection.DOWN);  // other way
		//}
				
		if( this.nbt != null ) {
			try {
			NBTTagCompound nbtTag;
			// read in nbtTag from this.nbt ... very version dependent
			nbtTag = (NBTTagCompound) JsonToNBT.func_150315_a(this.nbt);
			nbtTag.setInteger("x", blockPos.posX);
			nbtTag.setInteger("y", blockPos.posY);
			nbtTag.setInteger("z", blockPos.posZ);
			TileEntity te = TileEntity.createAndLoadEntity(nbtTag);
			world.setTileEntity(blockPos.posX,blockPos.posY,blockPos.posZ,te);
			} catch( Exception e ) {
				CheshireCat.logger.error("Unable to parse NBT '"+this.nbt+"': "+e.getMessage());		
				return false;
			}
		}

		return true;
	}
	
	@Override
	public String asText() {
		String s = "BLOCK ";
		if( block == null) {
			return "# BLOCK undefined ";
		}
		if(soft) { 
			s += "~"; 
		}
		s += Block.blockRegistry.getNameForObject(block);
		if(dv>0) {
			s+="("+dv+")";
		}
		s += " " + location.posX + "," + location.posY + "," + location.posZ;
		if( nbt != null ) {
			s += " " + nbt;
		}
		return s;
	}
}
