package org.steveshipway.cheshirecat.directive;

import java.util.regex.Matcher;
import java.util.regex.Pattern;

import org.steveshipway.cheshirecat.CheshireCat;

import net.minecraft.util.ChunkCoordinates;
import net.minecraft.world.World;

public class CallDirective extends Directive {
	private String blueprint = null;
	private ChunkCoordinates location;
	private int rotation = 0;
	
	public CallDirective(String blueprint, ChunkCoordinates offset, int r) {
		this.blueprint = blueprint;
		this.location = offset;
		this.rotation = r & 3;
	}
	public CallDirective(String definition,String myname) {
    	Pattern p = Pattern.compile("(?i)^(?:\\s*call)\\s*(\\w+)\\s+(-?\\d+)[\\s,]+(-?\\d+)[\\s,]+(-?\\d+)\\s*(\\d*)");
    	Matcher m = p.matcher(definition);
    	if( m.find() ) {
    		this.blueprint = m.group(1);
    		this.location = new ChunkCoordinates();
    		try { 
    		this.location.posX = Integer.parseInt( m.group(2) );
    		this.location.posY = Integer.parseInt( m.group(3) );
    		this.location.posZ = Integer.parseInt( m.group(4) );
    		} catch( Exception e ) { 
    			this.error = true; 
    			CheshireCat.logger.error("Invalid offset specified in call directive: "+definition);
    		}
    		if ( m.group(5) != "" ) {
    			try { this.rotation = Integer.parseInt( m.group(5) ) & 3; } catch( Exception e ) { }
    		}
    		if(! this.blueprint.equals(myname) && ! CheshireCat.portfolio.defines(this.blueprint) ) {
    			this.error = true;
    			CheshireCat.logger.error("Invalid blueprint name ["+this.blueprint+"] in definition!");
    		}
    	}
	}
	
	@Override
	public boolean build(World world, ChunkCoordinates location, int r) {
		ChunkCoordinates nc;
		int nr;
		if( blueprint == null ) {
			return false;
		}
		if( this.error ) {
			CheshireCat.logger.error("Error in call definition.  Cannot call ["+blueprint+"].");
			return false;
		}
		nc = rotateAndAdd( this.location, r, location );
		nr = ( this.rotation + r ) & 3;
		// CheshireCat.logger.info("Calling ["+blueprint+"] at location "+nc.posX+","+nc.posY+","+nc.posZ);
		
		return CheshireCat.portfolio.build(world,blueprint,nc,nr);
	}
	
	@Override
	public String asText() {
		return (this.error?"# ":"")+"CALL "+blueprint+" "+location.posX+", "+location.posY+", "+location.posZ+" "+rotation;
	}
}
