package org.steveshipway.cheshirecat.item;

import java.util.List;

import net.minecraft.item.Item;

import org.steveshipway.cheshirecat.Blueprint;
import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.lib.Constants;

import cpw.mods.fml.common.registry.GameRegistry;
import net.minecraft.creativetab.CreativeTabs;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.Item;
import net.minecraft.item.ItemStack;
import net.minecraft.nbt.NBTTagCompound;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.ChunkCoordinates;
import net.minecraft.util.EnumChatFormatting;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public class ItemWand extends Item {
	public static String name = "wand";
	
    public ItemWand(){
    	setUnlocalizedName(Constants.MODID + "_" + name);
    	setTextureName(Constants.MODID + ":" + name);
    	setCreativeTab(CreativeTabs.tabTools);
    	setMaxStackSize(1);
    	setHarvestLevel("wand",3);
    	GameRegistry.registerItem(this, name);
    }
    
    public boolean onItemUse(ItemStack item, EntityPlayer player, World world,
    	// which block was in the target when clicked
    	int posx, int posy, int posz, 
    	// where on the target block was clicked (0.0-1.0)
    	int side, float blockx, float blocky, float blockz)
    {
    	String blueprint = item.stackTagCompound.getString("blueprint");
        if(world.isRemote) {
    	 	// can't do anything here as we don't own the world (client side)
            return false;
        }
        if( blueprint == null || blueprint.matches("") ) {
    		// no blueprint is set...
    		player.addChatMessage(new ChatComponentText("Wand is not yet enchanted"));
    		return true;
    	}
   	    // activate the wand!
        // world.spawnEntityInWorld(new EntityLightningBolt(world, x, y, z));
        // we look at what we're pointing at.  The base coord is on top of the block if we're pointing 
        // at the top face, or in front if we're pointing at the side.
        ChunkCoordinates target = new ChunkCoordinates();
	    int rotation = MathHelper.floor_double((double)(player.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
        target.posX = posx;
        target.posY = posy;
        target.posZ = posz;
        if( side == 0 ) {
        	target.posY -= 1;
        } else if ( side ==  1) {
        	target.posY += 1;
        } else if ( side == 2 ) {
        	target.posZ -= 1;
        } else if ( side == 3 ) {
        	target.posZ += 1;
        } else if ( side == 4 ) {
        	target.posX -= 1;
        } else if ( side == 5 ) {
        	target.posX += 1;
        }
		//player.addChatMessage(new ChatComponentText("Building on side "+side+" when facing direction "+rotation));
        
        if( CheshireCat.portfolio.build(world, blueprint, target, rotation) ) {
    		player.addChatMessage(new ChatComponentText("Build was successful"));
        } else {
    		player.addChatMessage(new ChatComponentText("Build failed!"));
        }
        return true;
    }
    
    public void onCreated(ItemStack itemStack, World world, EntityPlayer player) {
        itemStack.stackTagCompound = new NBTTagCompound();
        itemStack.stackTagCompound.setString("blueprint", "");
    }
    
    @Override
    public boolean isItemTool(ItemStack p_77616_1_)  {
    	return true;
    }
    
    @Override
    public boolean canItemEditBlocks() {
    	return true;
    }
    
    public void addInformation(ItemStack par1ItemStack, EntityPlayer par2EntityPlayer, List par3List, boolean par4) {
    	NBTTagCompound nbt = par1ItemStack.stackTagCompound;
    	if( nbt == null ) {
    		par1ItemStack.stackTagCompound = new NBTTagCompound();
    		par1ItemStack.stackTagCompound.setString("blueprint", "");
    		par3List.add(EnumChatFormatting.RED + "Not initialised");
    		return;
    	}
    	String blueprint = nbt.getString("blueprint");
    	if( blueprint != null && !blueprint.matches("")) {
    		Blueprint bp;
    		bp = CheshireCat.portfolio.blueprint(blueprint);
    		par3List.add(EnumChatFormatting.GREEN + "Creates: " + blueprint);
    		if( bp != null ) {
    			par3List.add(EnumChatFormatting.AQUA + CheshireCat.portfolio.blueprint(blueprint).helpText() );
    		} else {
        		par3List.add(EnumChatFormatting.RED + "INVALID BLUEPRINT" );    		
    		}
    	} else {
    		par3List.add(EnumChatFormatting.AQUA + "Not yet enchanted");    	
    		//par3List.add(EnumChatFormatting.AQUA + "Use /wand set <blueprint>");    	
    	}
    }
    
    public int getItemEnchantability()
    {
        return 50; // it's a wand...
    }
    
}
