package org.steveshipway.cheshirecat;

import java.io.File;
import java.util.HashMap;
import java.util.Map;

import net.minecraft.block.BlockRailBase.EnumRailDirection;
import net.minecraft.block.properties.*;
import net.minecraft.block.state.*;
import net.minecraft.init.Blocks;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumFacing.Axis;

import org.apache.logging.log4j.Logger;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraftforge.common.MinecraftForge;
import net.minecraftforge.fml.common.Mod;
import net.minecraftforge.fml.common.Mod.EventHandler;
import net.minecraftforge.fml.common.Mod.Instance;
import net.minecraftforge.fml.common.Mod.Metadata;
import net.minecraftforge.fml.common.ModMetadata;
import net.minecraftforge.fml.common.SidedProxy;
import net.minecraftforge.fml.common.event.FMLInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPostInitializationEvent;
import net.minecraftforge.fml.common.event.FMLPreInitializationEvent;
import net.minecraftforge.fml.common.event.FMLServerStartingEvent;

@Mod(modid = Constants.MODID, name = Constants.MODNAME, version = Constants.VERSION,
		useMetadata = true)
public class CheshireCat
{    
   
	@Metadata(value = Constants.MODID)
	public static ModMetadata metadata;
	
	@SidedProxy(clientSide = "org.steveshipway.cheshirecat.ClientProxy", 
			serverSide = "org.steveshipway.cheshirecat.CommonProxy")
	public static CommonProxy proxy;

	// scan for all defined blueprints
	public static Portfolio portfolio;
	public static String bppath = null;
	public static Logger logger;
	
	public static void loadPortfolio() {
    	portfolio.load( bppath );
	}
	
    @Mod.EventHandler
    public void preInit(FMLPreInitializationEvent event) {
    	logger = event.getModLog();
    	logger.info("Starting Cheshire Cat Mod : Ver " + Constants.VERSION);
    	portfolio = new Portfolio(logger);
    	bppath = event.getModConfigurationDirectory() + File.separator + "bp";
    	File bpDir = new File(bppath);
    	if( !bpDir.exists() ) { bpDir.mkdir(); }
    	loadPortfolio();
    	proxy.preInit(event);
    }
 
    @Mod.EventHandler
    public void init(FMLInitializationEvent event) {
    	proxy.init(event);
    	// special client renderers
    	proxy.registerRenderers();
    }
 
    @Mod.EventHandler
    public void postInit(FMLPostInitializationEvent event) {
    	proxy.postInit(event);
    }
    
    @Mod.EventHandler
    public void serverLoad(FMLServerStartingEvent event)
    {
        logger.info("Loading server console commands.");
        proxy.serverLoad(event);
    }
    
    public static final PropertyDirection FACING = PropertyDirection.create("facing");
    public static final PropertyDirection AXIS = PropertyDirection.create("axis");
    public static final PropertyInteger ROTATION = PropertyInteger.create("rotation",0,15);
    public static final PropertyBool NORTH = PropertyBool.create("north");
    public static final PropertyBool SOUTH = PropertyBool.create("south");
    public static final PropertyBool EAST  = PropertyBool.create("east");
    public static final PropertyBool WEST  = PropertyBool.create("west");
    public static final PropertyDirection SHAPE = PropertyDirection.create("shape");
    public static IBlockState rotateState( IBlockState blockstate, int r ) {
    	EnumFacing facing;
    	EnumRailDirection raildirection;
    	Axis axis;
    	Integer rotation;
    	IBlockState newstate = blockstate;		
    	Map props = blockstate.getProperties();
    	// rotation for most blocks
    	if( props.containsKey(FACING) ) {      
    		facing = (EnumFacing) blockstate.getValue(FACING);
//            logger.info("Rotating a facing."+facing.toString()+" + "+r);
    		if( r == 1) {
        		facing = facing.rotateY();
    		} else if( r == 2) {
    			facing = facing.rotateY();
    			facing = facing.rotateY();
    		} else if(r == 3) {
    			facing = facing.rotateYCCW();
    		}
    		newstate = blockstate.withProperty(FACING, facing);
    		return newstate;
    	}
    	// rotation for logs, hay bales
    	if( props.containsKey(AXIS) ) {      
    		axis = (Axis) blockstate.getValue(AXIS);
//            logger.info("Rotating an axis."+axis.toString());
    		if( r == 1 || r == 3) {
    			if( axis == Axis.X ) {
    				axis = Axis.Z;
    			} else if( axis == Axis.Z ) {
    				axis = Axis.X;
    			}
    		}
    		newstate = blockstate.withProperty(AXIS, axis);
    		return newstate;
    	}
    	// rotation for banners and signs
    	if( props.containsKey(ROTATION) ) {      
    		newstate = blockstate;
    		rotation = (Integer) blockstate.getValue(ROTATION);
    		rotation = rotation + 2; 
    		if( rotation > 15 ) { rotation -= 16; }
    		newstate = blockstate.withProperty(ROTATION, rotation);
    		return newstate;
    	}
    	// rotation for vines
    	if( props.containsKey(NORTH) ) {      
    		int ir = r;
    		Boolean north = (Boolean) blockstate.getValue(NORTH);
    		Boolean south = (Boolean) blockstate.getValue(SOUTH);
    		Boolean east  = (Boolean) blockstate.getValue(EAST);
    		Boolean west  = (Boolean) blockstate.getValue(WEST);
    		Boolean t;
    		while( ir > 0 ) {
    			ir -= 1;
    			t = east; east = north; north = west; west = south; south = t;
    		}
    		newstate = blockstate.withProperty(NORTH, north).withProperty(SOUTH, south).withProperty(EAST, east).withProperty(WEST, west);
    		return newstate;
    	}
    	// rotation for train tracks; very weird
    	if( props.containsKey(SHAPE) ) {      
    		int ir = r;
    		HashMap<EnumRailDirection,EnumRailDirection> railmap = new HashMap<EnumRailDirection,EnumRailDirection>();
    		railmap.put(EnumRailDirection.NORTH_SOUTH,EnumRailDirection.EAST_WEST);
    		railmap.put(EnumRailDirection.EAST_WEST,EnumRailDirection.NORTH_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_NORTH,EnumRailDirection.ASCENDING_EAST);
    		railmap.put(EnumRailDirection.ASCENDING_EAST,EnumRailDirection.ASCENDING_SOUTH);
    		railmap.put(EnumRailDirection.ASCENDING_SOUTH,EnumRailDirection.ASCENDING_WEST);
    		railmap.put(EnumRailDirection.ASCENDING_WEST,EnumRailDirection.ASCENDING_NORTH);
    		railmap.put(EnumRailDirection.NORTH_EAST,EnumRailDirection.SOUTH_EAST);
    		railmap.put(EnumRailDirection.SOUTH_EAST,EnumRailDirection.SOUTH_WEST);
    		railmap.put(EnumRailDirection.SOUTH_WEST,EnumRailDirection.NORTH_WEST);
    		railmap.put(EnumRailDirection.NORTH_WEST,EnumRailDirection.NORTH_EAST);

    		raildirection = (EnumRailDirection) blockstate.getValue(SHAPE);
    		while( ir > 0 ) {
    			ir -= 1;
    			raildirection = railmap.get(raildirection);
    		}
    		newstate = blockstate.withProperty(SHAPE, raildirection);
    		return newstate;
    	}
    	// rotation for mushroom blocks; really weird
    	// XXXX
		return blockstate;
    }
}
