package org.steveshipway.cheshirecat.cmd;

import java.io.File;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import org.steveshipway.cheshirecat.CheshireCat;
import org.steveshipway.cheshirecat.ChunkCoordinates;
import org.steveshipway.cheshirecat.item.ItemWand;
import org.steveshipway.cheshirecat.lib.Constants;

import net.minecraft.client.Minecraft;
import net.minecraft.command.ICommand;
import net.minecraft.command.ICommandSender;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.item.ItemStack;
import net.minecraft.server.MinecraftServer;
import net.minecraft.util.BlockPos;
import net.minecraft.util.ChatComponentText;
import net.minecraft.util.MathHelper;
import net.minecraft.world.World;

public class BlueprintsCommand implements ICommand {
	  private List aliases;
	  public BlueprintsCommand()
	  {
	    this.aliases = new ArrayList();
	    this.aliases.add("blueprints");
	    this.aliases.add("bp");
	  }

	  @Override
	  public String getName()
	  {
	    return "blueprints";
	  }

	  @Override
	  public String getCommandUsage(ICommandSender icommandsender)
	  {
	    return "blueprints [list|build <blueprint>|info <blueprint>|save <blueprint>|reload]";
	  }

	  @Override
	  public List getAliases()
	  {
	    return this.aliases;
	  }

	  @Override
	  public void execute(ICommandSender icommandsender, String[] astring)
	  {
		  EntityPlayer player;
		  World world;
          
          if(icommandsender instanceof EntityPlayer){
                  player = (EntityPlayer)icommandsender;
    	    	  world = player.worldObj;
          } else {
                  icommandsender.addChatMessage(new ChatComponentText("Player Only Command"));
                  return;
          }
	      if(astring.length < 1 ) {
	    	  player.addChatMessage(new ChatComponentText("Please specify 'list', 'info', 'print', 'build', 'save' or 'reload'"));
	    	  return;
	      }
    	  CheshireCat.logger.info("Processing command ["+astring[0]+"]");
	      if( astring[0].startsWith("list") ) {
	    	  List helptext = CheshireCat.portfolio.help();
	    	  Iterator i = helptext.iterator();
	    	  while( i.hasNext() ) {
	    		  String s = (String) i.next();
		    	  player.addChatMessage(new ChatComponentText(s));
	    	  }
	    	  return;
	      }
	      if( astring[0].startsWith("reload") ) {
	    	  player.addChatMessage(new ChatComponentText("Reloading blueprints..."));
	    	  CheshireCat.loadPortfolio();
	    	  player.addChatMessage(new ChatComponentText("Complete"));
	    	  return;
	      }
	      if( astring[0].startsWith("copy") ) {
	    	  int copyopts = 0;
		      if( astring.length < 8 ) {
		    	  player.addChatMessage(new ChatComponentText("Must specify a new blueprint name, and two sets of coordinates x y z"));
		    	  return;	    	  	    	  
		      }
			  if( CheshireCat.portfolio.defines(astring[1]) ) {
		    	  player.addChatMessage(new ChatComponentText("Blueprint name '" + astring[1] + "' already exists."));
		    	  return;	    	  
		      }
	    	  ChunkCoordinates from = new ChunkCoordinates();
	    	  ChunkCoordinates to   = new ChunkCoordinates();
	    	  try {
	    		  from.posX = Integer.parseInt(astring[2]);
	    		  from.posY = Integer.parseInt(astring[3]);
	    		  from.posZ = Integer.parseInt(astring[4]);
	    		  to.posX = Integer.parseInt(astring[5]);
	    		  to.posY = Integer.parseInt(astring[6]);
	    		  to.posZ = Integer.parseInt(astring[7]);
	    	  } catch( Exception e ) {
		    	  player.addChatMessage(new ChatComponentText("Cannot parse coordinates.  Must be numeric integers."));
	    		  return;
	    	  }
	    	  try {
	    		  copyopts = Integer.parseInt(astring[8]);
	    	  } catch ( Exception e ) { copyopts = 0; }
	    	  player.addChatMessage(new ChatComponentText("Creating new blueprint "+astring[1]));
	    	  // last option is a flag; we'll use this to pass if we should copy nonsolid blocks (default no) and other options
	    	  if(CheshireCat.portfolio.copy(world, astring[1], from, to, copyopts) != null) {
		    	  player.addChatMessage(new ChatComponentText("Created successfully"));
	    	  } else {
		    	  player.addChatMessage(new ChatComponentText("Unable to copy block pattern"));
	    	  }
	    	  return;
	      }

	      // now we can test for a valid blueprint
	      if( astring.length < 2 ) {
	    	  player.addChatMessage(new ChatComponentText("Must specify blueprint name."));
	    	  return;	    	  	    	  
	      }
		  if( ! CheshireCat.portfolio.defines(astring[1]) ) {
	    	  player.addChatMessage(new ChatComponentText("Invalid blueprint name '" + astring[1] + "'"));
	    	  return;	    	  
	      }
	      if( astring[0].startsWith("info") ) {
	    	  String h = CheshireCat.portfolio.blueprint(astring[1]).helpText();
	    	  player.addChatMessage(new ChatComponentText(h));
	    	  return;
	      }
	      if( astring[0].startsWith("print") ) {
	    	  String h = CheshireCat.portfolio.blueprint(astring[1]).asText();
	    	  player.addChatMessage(new ChatComponentText(h));
	    	  return;
	      }
	      if( astring[0].startsWith("del") ) {
	    	  if( CheshireCat.portfolio.delete(astring[1]) ) {
	    		  player.addChatMessage(new ChatComponentText("Deleted blueprint '"+astring[1]+"' from memory."));
	    	  } else {
	    		  player.addChatMessage(new ChatComponentText("Unable to delete!"));	    		  
	    	  }
	    	  return;
	      }
	      if( astring[0].startsWith("build") ) {
		      int rotation = MathHelper.floor_double((double)(player.rotationYaw * 4.0F / 360.0F) + 0.5D) & 3;
		      //ChunkCoordinates location = player.playerLocation;
		      BlockPos bp = icommandsender.getPosition();
		      ChunkCoordinates location = new ChunkCoordinates(bp);
		      //location.posY += player.eyeHeight;
		      if( CheshireCat.portfolio.build(world,astring[1], location, rotation) ) {
		    	  player.addChatMessage(new ChatComponentText("Build successful"));
		      } else {
		    	  player.addChatMessage(new ChatComponentText("Build failed!"));
		      }
	    	  return;
	      }
	      if( astring[0].startsWith("save") ) {
	    	  player.addChatMessage(new ChatComponentText("Saving blueprint "+astring[1]));
	    	  if(CheshireCat.portfolio.save(astring[1])) {
		    	  player.addChatMessage(new ChatComponentText("Written successfully"));
	    	  } else {
		    	  player.addChatMessage(new ChatComponentText("Save failed!"));
	    	  }
	    	  return;
	      }

    	  player.addChatMessage(new ChatComponentText("Invalid command : " + astring[0]));
	      return;
	  }

	  @Override
	  public boolean canCommandSenderUse(ICommandSender icommandsender)
	  {
		  // is the player in create mode or holding the wand?
		  EntityPlayer player;
          
          if(icommandsender instanceof EntityPlayer){
                  player = (EntityPlayer)icommandsender;
          } else {
                  icommandsender.addChatMessage(new ChatComponentText("Player Only Command"));
                  return false;
          }
          if( player.capabilities.isCreativeMode ) {
        	  return true;
          }
          ItemStack item = player.getCurrentEquippedItem();
          if (item != null && item.getUnlocalizedName().endsWith(Constants.MODID + "_" + ItemWand.name) ) {
        	  return true;
          }
          
		  return false;
	  }

	  @Override
	  public List addTabCompletionOptions(ICommandSender icommandsender,
	      String[] astring, BlockPos bp)
	  {
		  List l = null;
		  if( astring.length == 0 ) {
			  l = new ArrayList();
			  l.add("list");
			  l.add("build");
			  l.add("info");
			  return l;
		  }
		  if( astring[0] == "list" ) {
			  // should add the list of blueprints to them...
			  return CheshireCat.portfolio.list(astring[1]);
		  }
		  if( astring.length == 1 && "list".startsWith( astring[0]) ) {
			  l = new ArrayList();
			  l.add("list");
			  return l;
		  }
		  return null;
	  }

	  @Override
	  public boolean isUsernameIndex(String[] astring, int i)
	  {
		  // none of the parameters are usernames
   	      return false;
	  }

	  @Override
	  public int compareTo(Object o)
	  {
	    return 0;
	  }
}
